﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Programmable {
  [Item("PermutationEncoding", "Describes a permutation encoding.")]
  [StorableClass]
  public class PermutationEncoding : Encoding {
    [Storable]
    private IntValue length;
    public IntValue Length {
      get { return length; }
      set {
        if (length == value) return;
        length = value;
        OnParameterConfigurationChanged();
      }
    }

    [Storable]
    private PermutationType type;
    public PermutationType Type {
      get { return type; }
      set {
        if (type == value) return;
        type = value;
        OnParameterConfigurationChanged();
      }
    }

    [StorableConstructor]
    protected PermutationEncoding(bool deserializing) : base(deserializing) { }
    protected PermutationEncoding(PermutationEncoding original, Cloner cloner)
      : base(original, cloner) {
      length = cloner.Clone(original.length);
      type = cloner.Clone(original.type);
    }
    public PermutationEncoding(string name, int length, PermutationTypes type)
      : base(name) {
      this.length = new IntValue(length);
      this.type = new PermutationType(type);
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new PermutationEncoding(this, cloner);
    }

  }
}
