﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Programmable {
  [Item("MultiEncoding Crossover", "Applies different crossovers to cross a multiencoding.")]
  [StorableClass]
  public class MultiEncodingCrossover : MultiOperator<ICrossover>, ICrossover {

    [StorableConstructor]
    protected MultiEncodingCrossover(bool deserializing) : base(deserializing) { }
    protected MultiEncodingCrossover(MultiEncodingCrossover original, Cloner cloner)
      : base(original, cloner) { }
    public MultiEncodingCrossover() {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new MultiEncodingCrossover(this, cloner);
    }

    public override IOperation InstrumentedApply() {
      var baseOperations = base.InstrumentedApply();
      return new OperationCollection(new[] { baseOperations }.Concat(Operators.Select(x => ExecutionContext.CreateOperation(x))));
    }
  }
}
